/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (C) 2006   SEGGER Microcontroller Systeme GmbH               *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.32a                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for RENESAS H83069 and IAR compiler)
Purpose : LED interface for EDK3069 starterkit
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** SFRs used for LED-Port **************************************/

/****** Port A data direction register */
#define PADDR   (*(volatile unsigned char __data32 *) (0xFFFEE009))
/****** Port A data register           */
#define PADR    (*(volatile unsigned char __data32 *) (0xFFFFFFD9))

#define LEDS_USED 1
#define LEDPORT PADR
#define LED0_BIT  2
#define LED1_BIT  3

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PADDR    |=  ((1 << LED0_BIT) | (1 << LED1_BIT));    // Enable LED output ports
  LEDPORT  &= ~((1 << LED0_BIT) | (1 << LED1_BIT));    // Switch off both LEDs
}

/*********************************************************************
*
*       LED switching routines
*/
void BSP_SetLED(int Index) {
  LEDPORT  |= (1 << (LED0_BIT + Index));    // Switch on LED
}

void BSP_ClrLED(int Index) {
  LEDPORT  &= ~(1 << (LED0_BIT + Index));   // Switch off LED
}

void BSP_ToggleLED (int Index) {
  LEDPORT  ^= (1 << (LED0_BIT + Index));    // Toggle LED
}

/****** End of File *************************************************/
