/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (C) 2006   SEGGER Microcontroller Systeme GmbH               *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.32a                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_H83069.c   (for RENESAS H83069 and IAR compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef OS_FSYS                     /* CPU main clock frequncy      */
  #define OS_FSYS 22118400L
#endif

#ifndef OS_PCLK_TIMER               /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER OS_FSYS     /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef OS_PCLK_UART                /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_FSYS      /* Normally derived from CPU    */
#endif                              /* clock                        */

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 1 per default
*       and supports UART0, UART1 and UART2
*/

#ifndef OS_UART
  #define OS_UART 1
#endif
#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

/*********************************************************************
*
*       SFR definition of sfrs used in RTOSInit.c
*
*       We define sfrs here to be independend of any i/o header file
*/

#define SYSCR     (*(volatile OS_U8 __data32 *) (0xFFFEE012)) /* System control            */
#define IPRA      (*(volatile OS_U8 __data32 *) (0xFFFEE018)) /* Interrupt control A       */
#define IPRB      (*(volatile OS_U8 __data32 *) (0xFFFEE019)) /* Interrupt control B       */

/****** SRFs used for embOS timer *************************************************/

#define TPU_IPR     IPRA
#define TPU_IPR_BIT 0

#define TPU_TSTR  (*(volatile OS_U8  __data32 *) (0xFFFFFF60)) /* Timer start               */
#define TPU_TMDR  (*(volatile OS_U8  __data32 *) (0xFFFFFF62)) /* Timer mode register       */
#define TPU_TISRA (*(volatile OS_U8  __data32 *) (0xFFFFFF64)) /* Timer interrupt status A  */
#define TPU_TISRB (*(volatile OS_U8  __data32 *) (0xFFFFFF65)) /* Timer interrupt status B  */

#define TPU_TCR   (*(volatile OS_U8  __data32 *) (0xFFFFFF78)) /* Timer control      ch 2   */
#define TPU_TIOR  (*(volatile OS_U8  __data32 *) (0xFFFFFF79)) /* Timer I/O ctrl H   ch 2   */
#define TPU_TCNT  (*(volatile OS_U16 __data32 *) (0xFFFFFF7A)) /* Timer count reg.   ch 2   */
#define TPU_TCNTH (*(volatile OS_U8  __data32 *) (0xFFFFFF7A)) /* Timer count high   ch 2   */
#define TPU_TCNTL (*(volatile OS_U8  __data32 *) (0xFFFFFF7B)) /* Timer count low    ch 2   */
#define TPU_TGRA  (*(volatile OS_U16 __data32 *) (0xFFFFFF7C)) /* Timer general regA ch 2   */
#define TPU_TGRAH (*(volatile OS_U8  __data32 *) (0xFFFFFF7C)) /* Timer general AH   ch 2   */
#define TPU_TGRAL (*(volatile OS_U8  __data32 *) (0xFFFFFF7D)) /* Timer general AL   ch 2   */

#define IMIA2               32           /* Timer interupt vector number          */
#define TPU_INTVEC          IMIA2
#define TPU_TISR            TPU_TISRA
#define TPU_INT_PENDING_BIT 2

/****** SFRs used for UART ********************************************************/

#if ((OS_UART == 0) || (OS_UART == 1) || (OS_UART == 2))
  #define OS_UART_USED 1
  #if (OS_UART == 0)
    #define UART_BASE 0xFFFFFFB0
    #define SCI_INTVEC_ERR  52
    #define SCI_INTVEC_RX   53
    #define SCI_INTVEC_TX   54
    #define SCI_IPR_REG     IPRB
    #define SCI_IPR_BIT        3
  #elif (OS_UART == 1)
    #define UART_BASE 0xFFFFFFB8
    #define SCI_INTVEC_ERR  56
    #define SCI_INTVEC_RX   57
    #define SCI_INTVEC_TX   58
    #define SCI_IPR_REG     IPRB
    #define SCI_IPR_BIT        2
  #elif (OS_UART == 2)
    #define UART_BASE 0xFFFFFFC0
    #define SCI_INTVEC_ERR  60
    #define SCI_INTVEC_RX   61
    #define SCI_INTVEC_TX   62
    #define SCI_IPR_REG     IPRB
    #define SCI_IPR_BIT        1
  #endif
#else
  #define OS_UART_USED 0
#endif

#if OS_UART_USED
  #define SCI_SMR   (*(volatile OS_U8 __data32 *) (UART_BASE+0)) /* SCI Serial mode register    */
  #define SCI_BRR   (*(volatile OS_U8 __data32 *) (UART_BASE+1)) /* SCI Bit rate register       */
  #define SCI_SCR   (*(volatile OS_U8 __data32 *) (UART_BASE+2)) /* SCI Serial control register */
  #define SCI_TDR   (*(volatile OS_U8 __data32 *) (UART_BASE+3)) /* SCI Transmit data register  */
  #define SCI_SSR   (*(volatile OS_U8 __data32 *) (UART_BASE+4)) /* SCI Serial status register  */
  #define SCI_RDR   (*(volatile OS_U8 __data32 *) (UART_BASE+5)) /* SCI Receive data register   */
  #define SCI_SCMR  (*(volatile OS_U8 __data32 *) (UART_BASE+5)) /* SCI Smart card moderegister */
  #define SCI_RX_PENDING_BIT 6
  #define SCI_TX_PENDING_BIT 7
#endif


/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#define _RESET_INT_PENDING(reg, b) reg = (reg & ~(1 << b))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW
*
Initialize hardware required for embOS to run.
1. We switch CPU into interrupt mode 1 (2 interrupt priorities)
2. We initialize a timer for embOS with low interrupt priority.
   We use 16bit Counter 2 in output compare mode.
This function should work on any hardware, but may have to be modified,
if you decide to use an other timer.
*/

void OS_InitHW(void) {
  OS_DI();                            // Initially disable interrupts
  __or_ccr((1 << 7) | (1 << 6));      // Mask all interrupts
  TPU_IPR  &= ~(1 << TPU_IPR_BIT);    // Set timer interrupt to low priority
  TPU_TSTR &= ~(1 << 2);              // Stop counter 2
  TPU_TCR   = 0x00                    // Prescaler set to 1 (b2..b0: 0)
            | (0 << 3)
            | (0 << 4)                // Count on rising edge
            | (1 << 5);               // Clear TCNT on TRGA compare
  TPU_TIOR &= 0xF8;                   // No output for TRGA compare match
  TPU_TGRA  = OS_PCLK_TIMER/1000-1;   // This value is for 1000 Interrupts
                                      // per sec => 1ms tick
                                      // Initialize Time constant TGR
  TPU_TMDR &= ~((1 << 6) | (1 << 2)); // Set timer 2 mode (normal)
  TPU_TSTR |= (1 << 2);               // Start counter 2
  TPU_TISR |= (1 << 6);               // enable compare match A interrupt
  OS_COM_Init();                      // Initialize communication to embOSView
  SYSCR    &= ~(1 << 3);              // Clear UE bit to select interrupt mode 1
  __and_ccr(~(1 << 6));               // Enable high priority interrupts
  OS_RestoreI();                      // Restore low priority interrupt enable state
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*

 Please note:
 This is basically the "core" of the idle loop
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) { // Idle loop: No task is ready to exec
  for (;;);          // Nothing to do ... wait for an interrupt
}

/*********************************************************************
*
*       OS_GetTimeCcycles()
*
    This routine is required for task-info via embOSView
    or high precision time measurment functions.
    It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 Cnt;
  OS_U32 Time;

  Cnt = TPU_TCNT;
  Time = OS_Time;
  if(TPU_TISR & (1 << TPU_INT_PENDING_BIT)) { // Re-read if interrupt pending
    Time++;
    Cnt = TPU_TCNT;
  }
  return (OS_PCLK_TIMER/1000) * Time + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
Purpose:
   Convert Cycles into micro seconds.
   If your clock frequency is not a multiple of 1 MHz, you may have to
   modify this routine in order to get proper diagonstics.
Please note:
   This routine is required for profiling or high resolution time
   measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*       embOS timer interrupt handler
*/
#pragma vector=TPU_INTVEC
static __interrupt void OS_ISR_Tick(void) {
  _RESET_INT_PENDING(TPU_TISR, TPU_INT_PENDING_BIT);
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  OS_HandleTick();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView via UART
*
*/
#if OS_UART_USED
#define BRR_VAL ((OS_PCLK_UART+OS_BAUDRATE*16L)/(OS_BAUDRATE*32L) -1 )
#if BRR_VAL > 255
  #error "UART can not be used with standard intialization."
#endif

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  SCI_SCR &= ~((0<<5)     // Clear Tx enable
              |(0<<4));   // Clear Rx enable
  SCI_SCR  =   (0<<7)     // Clear Tx interrupt enable
              |(0<<6)     // Clear Rx interrupt enable
              |(0<<5)     // Clear Tx enable
              |(0<<4)     // Clear Rx enable
              |(0<<3)     // Clear multi processor mode
              |(0<<2)     // Clear Tx end interrupt enable
              |(0<<1)     // Set internal clock mode
              |(0<<0);    // SCLK output not used
  SCI_SMR  =   (0<<7)     // Select Async. mode
              |(0<<6)     // Select 8 data bits
              |(0<<5)     // Disable parity
              |(0<<4)     // Even parity (don't care)
              |(0<<3)     // Select 1 stop bit
              |(0<<2)     // Disable multi processor mode
              |(0<<1)     // Clock divisor 1
              |(0<<0);    // Clock divisor 1
  SCI_BRR  =  BRR_VAL;    // Set calculated divisor
  SCI_IPR_REG &=  ~(1<<SCI_IPR_BIT);  // Set low UART interrupt priority
  SCI_SCR |=   (1<<6)     // Enable Rx interrupt
              |(1<<5)     // Enable Tx
              |(1<<4);    // Enable Rx
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  SCI_TDR  = c;
  _RESET_INT_PENDING(SCI_SSR, SCI_TX_PENDING_BIT);
  SCI_SCR |= (1 << 7);    //  Enable Tx-Interrupt
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART
*/
#pragma vector=SCI_INTVEC_TX
static __interrupt void OS_ISR_tx(void) {
  _RESET_INT_PENDING(SCI_SSR, SCI_TX_PENDING_BIT);
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  if (OS_OnTx()) {        //  No more characters to send ?
    SCI_SCR &= ~(1 << 7); //  Disable Tx Interrupt
  }
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART
*/
#pragma vector=SCI_INTVEC_RX
static __interrupt void OS_ISR_rx(void) {
  _RESET_INT_PENDING(SCI_SSR, SCI_RX_PENDING_BIT);
  OS_EnterNestableInterrupt();  //  May be nested
  OS_EnterIntStack();
  OS_OnRx(SCI_RDR);
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_err() Rx Error interrupt handler for UART
*/
#pragma vector=SCI_INTVEC_ERR
static __interrupt void OS_ISR_err (void) {
  SCI_SSR = SCI_SSR & ~((5 << 1) | (4 << 1) | (3 << 1));
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/

#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }

#endif /* OS_UART_USED */

/*****  EOF  ********************************************************/
