/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (C) 2006   SEGGER Microcontroller Systeme GmbH               *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.32a                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : Board support package
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** SFRs used for LED-Port **************************************/

/****** Port E data direction register */
#define PEDDR   (*(volatile unsigned char __data32 *) (0xFFFFFE3D))
/****** Port E data register           */
#define PEDR    (*(volatile unsigned char __data32 *) (0xFFFFFF0D))

#define LED_DATA_PORT PEDR
#define LED_DDR_PORT  PEDDR
#define LED0_BIT  0
#define LED1_BIT  1

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  LED_DDR_PORT   |=  ((1 << LED0_BIT) | (1 << LED1_BIT));    // Enable LED output ports
  LED_DATA_PORT  &= ~((1 << LED0_BIT) | (1 << LED1_BIT));    // Switch off both LEDs
}

/*********************************************************************
*
*       LED switching routines
*/
void BSP_SetLED (int Index) {
  LED_DATA_PORT  |= (1 << ( Index + LED0_BIT));    // Switch on LED
}

void BSP_ClrLED(int Index) {
  LED_DATA_PORT  &= ~(1 << ( Index + LED0_BIT));   // Switch off LED
}

void BSP_ToggleLED(int Index) {
  LED_DATA_PORT  ^= (1 << ( Index + LED0_BIT));    // Toggle LED
}

/****** End of File *************************************************/
