/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (C) 2006   SEGGER Microcontroller Systeme GmbH               *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.32a                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_H8S2239.c   (for RENESAS H8S2239 and IAR compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef OS_FSYS                     /* CPU main clock frequncy      */
  #define OS_FSYS 14745600L
#endif

#ifndef OS_PCLK_TIMER               /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER OS_FSYS     /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef OS_PCLK_UART                /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_FSYS      /* Normally derived from CPU    */
#endif                              /* clock                        */

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/

#ifndef OS_UART
  #define OS_UART 0
#endif
#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

/*********************************************************************
*
*       SFR definition of sfrs used in RTOSInit.c
*
*       We define sfrs here to be independend of any i/o header file
*/

#define SYSCR     (*(volatile OS_U8 __data32 *) (0xFFFFFDE5)) /* System Control            */
#define MSTPCRA   (*(volatile OS_U8 __data32 *) (0xFFFFFDE8)) /* System Control            */
#define MSTPCRB   (*(volatile OS_U8 __data32 *) (0xFFFFFDE9)) /* System Control            */
#define MSTPCRC   (*(volatile OS_U8 __data32 *) (0xFFFFFDEA)) /* System Control            */

/****** SRFs used for embOS timer *************************************************/

#define TPU_TSTR  (*(volatile OS_U8 *) (0xFFFFFEB0)) /* Timer start               */

/****** sfr definition for TPU channel 2 ******************************************/
#define TPU_TCR_2  (*(volatile OS_U8  __data32 *) (0xFFFFFF30)) /* Timer control       ch 2 */
#define TPU_TMDR_2 (*(volatile OS_U8  __data32 *) (0xFFFFFF31)) /* Timer mode register ch 2 */
#define TPU_TIOR_2 (*(volatile OS_U8  __data32 *) (0xFFFFFF32)) /* Timer I/O ctrl      ch 2 */
#define TPU_TIER_2 (*(volatile OS_U8  __data32 *) (0xFFFFFF34)) /* Timer int enable    ch 2 */
#define TPU_TSR_2  (*(volatile OS_U8  __data32 *) (0xFFFFFF35)) /* Timer status        ch 2 */
#define TPU_TCNT_2 (*(volatile OS_U16 __data32 *) (0xFFFFFF36)) /* Timer count reg.    ch 2 */
#define TPU_TGRA_2 (*(volatile OS_U16 __data32 *) (0xFFFFFF38)) /* Timer general regA  ch 2 */

/****** Interrupt vector and sfr definition ***************************************/

#define TGI2A      44            /* Timer int vector number TPU ch 2 TRGA match   */
#define IPRG       (*(volatile OS_U8 __data32 *) (0xFFFFFEC6)) /* interrupt priority reg.G */

/****** Assign logical used sfr names to selcted timer ****************************/
#define TPU_CHANNEL 2
#define TPU_IER_BIT 0            /*  Interrupt on TRGA compare match       */
#define TPU_INT_PENDING_BIT 0    /*  TRGA compare interrupt condition      */
#define TPU_TIER   TPU_TIER_2
#define TPU_TCR    TPU_TCR_2
#define TPU_TIOR   TPU_TIOR_2
#define TPU_TCNT   TPU_TCNT_2
#define TPU_TGRA   TPU_TGRA_2
#define TPU_TMDR   TPU_TMDR_2
#define TPU_TISR   TPU_TSR_2     /*  Timer interrupt status register       */
#define TPU_IPR    IPRG          /*  Timer interrupt priority register     */
#define TPU_IPR_MASK 0xF0        /*  used to set timer interrupt priority  */
#define TPU_INTVEC TGI2A

/****** SFRs used for UART ********************************************************/

/****** Interrupt priority control register****************************************/

#define IPRJ       (*(volatile OS_U8 __data32 *) (0xFFFFFEC9)) /* interrupt priority reg.J */
#define IPRK       (*(volatile OS_U8 __data32 *) (0xFFFFFECA)) /* interrupt priority reg.K */
#define IPRO       (*(volatile OS_U8 __data32 *) (0xFFFFFECE)) /* interrupt priority reg.O */

#if ((OS_UART == 0) || (OS_UART == 1) || (OS_UART == 2) || (OS_UART == 3))
  #define OS_UART_USED 1
  #if (OS_UART == 0)
    #define UART_BASE 0xFFFFFF78
    #define SCI_INTVEC_ERR  80
    #define SCI_INTVEC_RX   81
    #define SCI_INTVEC_TX   82
    #define SCI_IPR         IPRJ
    #define SCI_IPR_MASK    0x0F
    #define SCI_MSTPCR   MSTPCRB
    #define SCI_MSTPCR_BIT     7
  #elif (OS_UART == 1)
    #define UART_BASE 0xFFFFFF80
    #define SCI_INTVEC_ERR  84
    #define SCI_INTVEC_RX   85
    #define SCI_INTVEC_TX   86
    #define SCI_IPR         IPRK
    #define SCI_IPR_MASK    0xF0
    #define SCI_MSTPCR   MSTPCRB
    #define SCI_MSTPCR_BIT     6
  #elif (OS_UART == 2)
    #define UART_BASE 0xFFFFFF88
    #define SCI_INTVEC_ERR  87
    #define SCI_INTVEC_RX   88
    #define SCI_INTVEC_TX   89
    #define SCI_IPR         IPRK
    #define SCI_IPR_MASK    0x0F
    #define SCI_MSTPCR   MSTPCRB
    #define SCI_MSTPCR_BIT     5
  #elif (OS_UART == 3)
    #define UART_BASE 0xFFFFFDD0
    #define SCI_INTVEC_ERR 120
    #define SCI_INTVEC_RX  121
    #define SCI_INTVEC_TX  122
    #define SCI_IPR         IPRO
    #define SCI_IPR_MASK    0xF0
    #define SCI_MSTPCR   MSTPCRC
    #define SCI_MSTPCR_BIT     7
  #endif
#else
  #define OS_UART_USED 0
#endif

#if OS_UART_USED
  #define SCI_SMR   (*(volatile OS_U8 __data32 *) (UART_BASE+0)) /* SCI Serial mode register    */
  #define SCI_BRR   (*(volatile OS_U8 __data32 *) (UART_BASE+1)) /* SCI Bit rate register       */
  #define SCI_SCR   (*(volatile OS_U8 __data32 *) (UART_BASE+2)) /* SCI Serial control register */
  #define SCI_TDR   (*(volatile OS_U8 __data32 *) (UART_BASE+3)) /* SCI Transmit data register  */
  #define SCI_SSR   (*(volatile OS_U8 __data32 *) (UART_BASE+4)) /* SCI Serial status register  */
  #define SCI_RDR   (*(volatile OS_U8 __data32 *) (UART_BASE+5)) /* SCI Receive data register   */
  #define SCI_SCMR  (*(volatile OS_U8 __data32 *) (UART_BASE+6)) /* SCI Smart card moderegister */
  #define SCI_RX_PENDING_BIT 6
  #define SCI_TX_PENDING_BIT 7
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#define _RESET_INT_PENDING(reg, b) reg = (reg & ~(1 << b))

/*********************************************************************
*
*       _ISR_TickHandler()
*
*       embOS timer interrupt handler
*/
static void _ISR_TickHandler(void) {
  _RESET_INT_PENDING(TPU_TISR, TPU_INT_PENDING_BIT);
  OS_HandleTick();
}  

#if OS_UART_USED
/*********************************************************************
*
*       _ISR_TxHandler()
*       Tx interrupt handler
*/
static _ISR_TxHandler (void) {
  _RESET_INT_PENDING(SCI_SSR, SCI_TX_PENDING_BIT);
  if (OS_OnTx()) {        //  No more characters to send ?
    SCI_SCR &= ~(1 << 7); //  Disable Tx Interrupt
  }
}

/*********************************************************************
*
*       _ISR_RxHandler()
*       Rx interrupt handler
*/
static void _ISR_RxHandler(void) {
  _RESET_INT_PENDING(SCI_SSR, SCI_RX_PENDING_BIT);
  OS_OnRx(SCI_RDR);
}

/*********************************************************************
*
*       _ISR_RxErrHandler()
*       Rx Error interrupt handler for UART
*/
static void _ISR_RxErrHandler(void) {
  SCI_SSR = SCI_SSR & ~((5 << 1) | (4 << 1) | (3 << 1));
}
#endif /* OS_UART_USED */

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW
*

Initialize hardware required for embOS to run.
1. We switch CPU into interrupt mode 1 (2 interrupt priorities)
2. We initialize a timer for embOS with low interrupt priority.
   We use 16bit Counter 5 in output compare mode.
This function should work on any hardware, but may have to be modified,
if you decide to use an other timer.
*/
void OS_InitHW(void) {
  __set_imask_ccr(1);                  // Disable all interrupts
  OS_DI();                             // Initially disable interrupts
  SYSCR    &= ~((1 << 5) | (1 << 4));  // Set interrupt mode 0
  SYSCR    |= (1 << 5);                // select interrupt mode 2
  MSTPCRA  &= ~(1 << 5);               // Release TPU stop mode
  TPU_TSTR &= ~(1 << TPU_CHANNEL);     // Stop counter
  TPU_TCR   = 0x00                     // Prescaler set to 1 (b2..b0: 0)
            | (0 << 3)
            | (0 << 4)                 // Count on rising edge
            | (1 << 5);                // Clear TCNT on TRGA compare
  TPU_TIOR &= 0xF0;                    // No output for TRGA compare match
  TPU_TGRA  = OS_PCLK_TIMER/1000-1;    // This value is for 1000 Interrupts
                                       // per sec => 1ms tick
                                       // Initialize Time constant TGR
  TPU_TMDR  = 0;                       // Set timer to normal mode
  TPU_TIER |= (1 << TPU_IER_BIT);      // enable compare match interrupt
  TPU_IPR  &= ~(TPU_IPR_MASK);         // Set timer int to lowest priority (1)
  TPU_IPR  |= (0x11 & TPU_IPR_MASK);   //
  TPU_TSTR |= (1 << TPU_CHANNEL);      // Start counter
                                       //
  OS_COM_Init();                       // Initialize communication to embOSView
  __set_imask_ccr(0);                  // reset i mask bit (important for interrupt control using embOS !)
  OS_RestoreI();                       // Restore low priority interrupt enable state
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*

 Please note:
 This is basically the "core" of the idle loop
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) { // Idle loop: No task is ready to exec
  for (;;);          // Nothing to do ... wait for an interrupt
}

/*********************************************************************
*
*       OS_GetTimeCcycles()
*
    This routine is required for task-info via embOSView
    or high precision time measurment functions.
    It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 Cnt;
  OS_U32 Time;

  Cnt = TPU_TCNT;
  Time = OS_Time;
  if(TPU_TISR & (1 << TPU_INT_PENDING_BIT)) { // Re-read if interrupt pending
    Time++;
    Cnt = TPU_TCNT;
  }
  return (OS_PCLK_TIMER/1000) * Time + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert Cycles into micro seconds.

   If your clock frequency is not a multiple of 1 MHz, you may have to
   modify this routine in order to get proper diagonstics.
Please note:
   This routine is required for profiling or high resolution time
   measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_FSYS/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*       embOS timer interrupt handler
*/
#pragma vector=TPU_INTVEC
static __interrupt void OS_ISR_Tick(void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  _ISR_TickHandler();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView via UART
*
*/
#if OS_UART_USED
#define BRR_VAL ((OS_PCLK_UART+OS_BAUDRATE*16L)/(OS_BAUDRATE*32L) -1 )
#if BRR_VAL > 255
  #error "UART can not be used with standard intialization."
#endif

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  SCI_MSTPCR &=  ~(1 << SCI_MSTPCR_BIT); // Enable SIO module (clear stop mode)
  SCI_SCR    &= ~((1 << 5)               // Clear Tx enable
                 |(1 << 4));             // Clear Rx enable
  SCI_SCR     =   (0 << 7)               // Clear Tx interrupt enable
                 |(0 << 6)               // Clear Rx interrupt enable
                 |(0 << 5)               // Clear Tx enable
                 |(0 << 4)               // Clear Rx enable
                 |(0 << 3)               // Clear multi processor mode
                 |(0 << 2)               // Clear Tx end interrupt enable
                 |(0 << 1)               // Set internal clock mode
                 |(0 << 0);              // SCLK output not used
  SCI_SMR     =   (0 << 7)               // Select Async. mode
                 |(0 << 6)               // Select 8 data bits
                 |(0 << 5)               // Disable parity
                 |(0 << 4)               // Even parity (don't care)
                 |(0 << 3)               // Select 1 stop bit
                 |(0 << 2)               // Disable multi processor mode
                 |(0 << 1)               // Clock divisor 1
                 |(0 << 0);              // Clock divisor 1
  SCI_BRR     =  BRR_VAL;                // Set calculated divisor
  SCI_IPR    &= ~(SCI_IPR_MASK);         // Set UART int to lowest priority (1)
  SCI_IPR    |=  (0x11 & SCI_IPR_MASK);
  SCI_SCR    |=   (1 << 6)               // Enable Rx interrupt
                 |(1 << 5)               // Enable Tx
                 |(1 << 4);              // Enable Rx
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  SCI_TDR  = c;
  _RESET_INT_PENDING(SCI_SSR, SCI_TX_PENDING_BIT);
  SCI_SCR |= (1<<7);    //  Enable Tx-Interrupt
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART
*/
#pragma vector=SCI_INTVEC_TX
static __interrupt void OS_ISR_tx (void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();  
  _ISR_TxHandler();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART
*/
#pragma vector=SCI_INTVEC_RX
static __interrupt void OS_ISR_rx(void) {
  OS_EnterNestableInterrupt();  //  May be nested
  OS_EnterIntStack();
  _ISR_RxHandler();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_err() Rx Error interrupt handler for UART
*/
#pragma vector=SCI_INTVEC_ERR
static __interrupt void OS_ISR_err(void) {
  OS_EnterInterrupt();
  _ISR_RxErrHandler();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/

#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }

#endif /* OS_UART_USED */

/*****  EOF  ********************************************************/
